/*
ESUP-portail is a french academic project developed under the GPL (General Public License) augmented according to the following :
A binary or source file developped by ESUP-portail can be used or compiled with products under Apache license.
The official english text of the GPL can be found here : http://www.gnu.org/licenses/gpl.html .
A non official french translation can be found here : http://www.linux-France.org/article/these/gpl.html .
The different kinds of licenses governing the products developed by the Apache foundation can be found here : http://www.apache.org/licenses .
It follows that you can as well as use download contents as well modify and redistribute them provided you respect the GPL terms.
Downloading and using such contents do not provide any guaranty.
Be sure that you have well understood the terms of the license before using the contents it covers.
The ESUP-portail distribution includes the following distributions :
    * UPortal : 
      software developed by JA-SIG (Java Architecture - Special Interest Group)
      You can find the license page here : http://mis105.udel.edu/ja-sig/uportal/license.html
    * CAS :
      SSO solution developed by Yale University
      You can find the project page here : http://www.yale.edu/tp/auth
    * Cocoon :
      XML framework distributed by the Apache foundation under Apache license;
      Please find the full text here : http://cocoon.apache.org/2.1/license.html
    * Mod_dav:
      A DAV module for Apache web server
      You can find the project page here : http://www.webdav.org/mod_dav
    * IMP :
      webmail from Horde application framework
      You can find the project page here : http://www.horde.org
    * . To be completed
*/ 

package org.esupportail.portal.channels.CTestMailToNormal;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.*;
import org.jasig.portal.utils.XSLT;
import org.xml.sax.ContentHandler;
import org.esupportail.portal.channels.CMailTo.Mail.*;
import org.esupportail.portal.channels.CMailTo.MailExceptions.MalFormedMailException;


public class CTestMailToNormal implements IChannel,ICacheable{

	private static final Log log = LogFactory.getLog(CTestMailToNormal.class);
	ChannelStaticData staticData = null;
	ChannelRuntimeData runtimeData = null;
	
	private int cacheTimeOut = 60;
	private IServant servant;
	private boolean servantAvailable = false;
	
	public CTestMailToNormal(){
	}
	
	
    /* (non-Javadoc)
     * @see org.jasig.portal.IChannel#setStaticData(org.jasig.portal.ChannelStaticData)
     */
    public void setStaticData(ChannelStaticData sd) throws PortalException {
        this.staticData = sd;
        
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.IChannel#setRuntimeData(org.jasig.portal.ChannelRuntimeData)
     */
    public void setRuntimeData(ChannelRuntimeData rd) throws PortalException {
        runtimeData = rd;
		
		if ( (!servantAvailable)
		     && (rd.getParameter("action")!=null) 
		     && (rd.getParameter("action").equals("mailTo"))
		   ){
		    /**/
		    servantAvailable = true;
		}
		
		
		if (servantAvailable){	
		    
		    if (rd.getParameter("case")!=null) {
		        if (rd.getParameter("case").equals("1")){
		            prepareCase1();
		        }    
			    else if (rd.getParameter("case").equals("2")){
		            prepareCase2();
		        }      
			    else if (rd.getParameter("case").equals("3")){
		            prepareCase3();
		        }    
		    }
		    
		    
		    try {
                Class servantClass = Class.forName("org.esupportail.portal.channels.CMailTo.CMailToServant");
                // Instanciation du servant
                Object servantObj = servantClass.newInstance();
                servant = (IServant)servantObj;
            } catch (ClassNotFoundException e) {
                log.error("CTestMailtoNormal::setRuntimeData:ClassNotFoundException : "+e.getMessage());
            } catch (InstantiationException e) {
            	log.error("CTestMailtoNormal::setRuntimeData:InstantiationException : "+e.getMessage());
            } catch (IllegalAccessException e) {
            	log.error("CTestMailtoNormal::setRuntimeData:IllegalAccessException : "+e.getMessage());
            } catch (Exception e) {
            	log.error("CTestMailtoNormal::setRuntimeData:Exception : "+e.getMessage());
            }

            //Passage des paramtres
            servant.setStaticData(staticData);
            servant.setRuntimeData(runtimeData);
            
            if (servant.isFinished()) {
                servantAvailable=false;
            }		    
		}
		
		if ( (!servantAvailable)
			     && (rd.getParameter("action")!=null) 
			     && (rd.getParameter("action").equals("sendMail"))
			   ){

				Mail myMail = new Mail();
				myMail.setFrom(staticData.getPerson().getAttribute("mail").toString());
				myMail.setTo("sgaudin@univ-nancy2.fr");
				myMail.setSubject("Test de mail");
				myMail.setContent("message envoy automatiquement");
				
				try {
				    myMail.send();
		        } catch (MalFormedMailException e) {
		        	rd.setParameter("errorSendMail","true");
		        	log.error("CTestMailToNormal::setRuntimeData : ",e);
		        }
			}
    }
    
    private void prepareCase1(){
        String[] att = new String[3];
        //destinataire
        att[0] = "sgaudin@univ-nancy2.fr";
        att[1] = "false";
        att[2] = "true";
        runtimeData.setParameterValues(Mail.TO,att);
    }
    
    private void prepareCase2(){
        String[] att = new String[3];
        //cc
        att = new String[3];
        att[0] = "";
        att[1] = "false";
        att[2] = "false";
        runtimeData.setParameterValues(Mail.CC,att);
        //bcc
        att = new String[3];
        att[0] = "";
        att[1] = "false";
        att[2] = "false";
        runtimeData.setParameterValues(Mail.BCC,att);
    }
    
    private void prepareCase3(){
        String[] att = new String[3];
        //destinataire
        att[0] = "sgaudin@univ-nancy2.fr";
        att[1] = "false";
        att[2] = "true";
        runtimeData.setParameterValues(Mail.TO,att);
        //cc
        att = new String[3];
        att[0] = "";
        att[1] = "false";
        att[2] = "false";
        runtimeData.setParameterValues(Mail.CC,att);
        //bcc
        att = new String[3];
        att[0] = "";
        att[1] = "false";
        att[2] = "false";
        runtimeData.setParameterValues(Mail.BCC,att);
        //sujet
        att = new String[3];
        att[0] = "Test de mail";
        att[1] = "true";
        att[2] = "true";
        runtimeData.setParameterValues(Mail.SUBJECT,att);	
        //sujet
        att = new String[3];
        att[0] = "Message  modifier";
        att[1] = "true";
        att[2] = "true";
        runtimeData.setParameterValues(Mail.CONTENT,att);	
        
    }
    
      
    
    /* (non-Javadoc)
     * @see org.jasig.portal.IChannel#receiveEvent(org.jasig.portal.PortalEvent)
     */
    public void receiveEvent(PortalEvent ev) {
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.IChannel#getRuntimeProperties()
     */
    public ChannelRuntimeProperties getRuntimeProperties() {
		return new ChannelRuntimeProperties();
    }

    
        
    /* (non-Javadoc)
     * @see org.jasig.portal.IChannel#renderXML(org.xml.sax.ContentHandler)
     */
    public void renderXML(ContentHandler out) throws PortalException {
    	
        
        if (servantAvailable){
            servant.renderXML(out);
        }
        else{
             myRenderXML(out);
        }
    }
        
   private void myRenderXML(ContentHandler out) throws PortalException {     
        XSLT xslt = null; 
    	
        xslt = new XSLT(this);
        StringBuffer xml = new StringBuffer();
        xml.append("<xml>");
        
        if ( (runtimeData.getParameter("errorSendMail")!=null)
        	&& (runtimeData.getParameter("errorSendMail").equals("true"))
			){
        	xml.append("<error id=\"sendMail\" />");
        }
        xml.append("</xml>");
        xslt.setXML(xml.toString());
        
        /*Positionnement de la feuille XSL*/
		try {
			xslt.setXSL("CTestMailtoNormal.xsl");
		} catch (PortalException e) {
			log.error("CTestMailtoNormal::renderXml() : " + e.toString());
		}
		 
            
		
		/*Envoie des paramtres au fichier XSL*/
		xslt.setStylesheetParameter("baseActionURL",runtimeData.getBaseActionURL());
		
		
		/*Rendu du canal*/
		xslt.setTarget(out);
		try {
		    xslt.transform();
		} catch (PortalException e) {
			log.error("CTestMailtoNormal::renderXml() : Error in parsing XML/XSL : "
					+ e.getMessage());
		}
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.ICacheable#generateKey()
     */
    public ChannelCacheKey generateKey() {
        ChannelCacheKey k = new ChannelCacheKey();
        StringBuffer sbKey = new StringBuffer(1024);
        sbKey.append(this.getClass().getName() + " : ");
        sbKey.append("userId:").append(staticData.getPerson().getID());        
        k.setKey(sbKey.toString());
        k.setKeyScope(ChannelCacheKey.INSTANCE_KEY_SCOPE);
        k.setKeyValidity(new Long(System.currentTimeMillis()));
		return k;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.ICacheable#isCacheValid(java.lang.Object)
     */
    public boolean isCacheValid(Object validity) {
		boolean cacheValid = false;
		
		if (servantAvailable) return (false);
		
		if ( (!servantAvailable) && (servant!=null) && (servant.isFinished()) ) return (false);
		
		if (validity instanceof Long) {
			Long oldtime = (Long) validity;
			// Si validity  moins de cacheTimeOut secondes le cache est valide
			if (System.currentTimeMillis() - oldtime.longValue()
				< cacheTimeOut *60 * 1000) {
				cacheValid = true;
			}
		}
		
		return cacheValid;
    }

}
